@@ -1,10 +1,12 @@
+use 5.006;
 use warnings;
 use strict;
 use Carp;
 
 package Template::Declare;
+use Template::Declare::Buffer;
 
-$Template::Declare::VERSION = "0.02";
+$Template::Declare::VERSION = "0.03";
 
 use base 'Class::Data::Inheritable';
 __PACKAGE__->mk_classdata('roots');
@@ -12,6 +14,7 @@ __PACKAGE__->mk_classdata('aliases');
 __PACKAGE__->mk_classdata('alias_prefixes');
 __PACKAGE__->mk_classdata('templates');
 __PACKAGE__->mk_classdata('private_templates');
+__PACKAGE__->mk_classdata('buffer_stack');
 
 
 __PACKAGE__->roots([]);
@@ -19,7 +22,9 @@ __PACKAGE__->aliases([]);
 __PACKAGE__->alias_prefixes({});
 __PACKAGE__->templates({});
 __PACKAGE__->private_templates({});
+__PACKAGE__->buffer_stack([]);
 
+__PACKAGE__->new_buffer_frame();
 
 
 =head1 NAME
@@ -161,19 +166,44 @@ sub init {
 
 }
 
+
+sub new_buffer_frame {
+    my $buffer = Template::Declare::Buffer->new();
+    unshift @{__PACKAGE__->buffer_stack}, $buffer;
+
+}
+
+sub end_buffer_frame {
+    shift @{__PACKAGE__->buffer_stack};
+}
+
+sub buffer {
+    unless (__PACKAGE__->buffer_stack->[0] ) { Carp::confess(__PACKAGE__."->buffer called with no buffer");}
+    return __PACKAGE__->buffer_stack->[0] ;
+}
+
+
 =head2 show TEMPLATE_NAME
 
-Call C<show> with a C<template_name> and C<Template::Declare> will render that template and return the content as a scalar.
+Call C<show> with a C<template_name> and C<Template::Declare> will
+render that template. Content generated by show can be accessed with
+the C<output> method if the output method you've chosen returns content
+instead of outputting it directly.
+
+(If called in scalar context, this method will also just return the
+content when available).
+
+
 
 =cut
 
 sub show {
     my $class = shift;
     my $template = shift;
-    Template::Declare::Tags::show($template);
-
+    return Template::Declare::Tags::show($template);
 }
 
+
 =head2 alias
 
  alias Some::Clever::Mixin under '/mixin';
@@ -191,16 +221,16 @@ sub alias {
 }
 
 
-=head2 import
+=head2 import_templates
 
 
- import Wifty::UI::something under '/something';
+ import_templates Wifty::UI::something under '/something';
 
 
 =cut
 
 
-sub import {
+sub import_templates {
     return undef if $_[0] eq 'Template::Declare';
     my $import_into      = caller(0);
     my $import_from_base = shift;
@@ -289,6 +319,11 @@ sub resolve_template {
     my $show_private  = shift || 0;
 
     foreach my $package ( reverse @{ Template::Declare->roots } ) {
+        unless ($package and $package->isa('Template::Declare')) {
+            warn "'@{[$package||'']} was listed as a Template::Declare root, but is not a Template::Declare subclass";
+            next;
+        }
+
         if ( my $coderef = $package->has_template( $template_name, $show_private ) ) {
             return $coderef;
         }
